﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.Statements;
using System.Windows.Media;

namespace Konverter.AttribValueConverters
{
    /// <summary>
    /// Konverter für einen in Xaml beschriebenen Brush in C#-Code.
    /// </summary>
    class AttBrushConverter
    {
        /// <summary>
        /// Benötigte Usings des Brushs.
        /// </summary>
        public static List<UsingStmt> Usings
        {
            get
            {
                return new List<UsingStmt>() {
                    new UsingStmt("System.Windows.Media")
                };
            }
        }

        /// <summary>
        /// Konvertiere den in Xaml definierten Brush in C#-Code.
        /// </summary>
        /// <param name="value">Xaml-Code des Brushs</param>
        /// <returns>C#-Code des Brushs</returns>
        public static string Convert(string value)
        {
            try
            {
                //RGB, ARGB -> ScRGB, ScARGB -> sonst "Red" usw.
                if (value.IndexOf("sc#") >= 0)
                {
                    BrushConverter brConv = new BrushConverter();
                    SolidColorBrush sldColorBrsh = (SolidColorBrush)brConv.ConvertFrom(value);

                    string a = sldColorBrsh.Color.ScA.ToString(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"));
                    string r = sldColorBrsh.Color.ScR.ToString(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"));
                    string g = sldColorBrsh.Color.ScG.ToString(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"));
                    string b = sldColorBrsh.Color.ScB.ToString(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"));

                    return Get_ScArgb_SolidColorBrush(a, r, g, b);
                }
                else if (value.IndexOf("#") >= 0)
                {
                    BrushConverter brConv = new BrushConverter();
                    SolidColorBrush sldColorBrsh = (SolidColorBrush)brConv.ConvertFrom(value);

                    int a = sldColorBrsh.Color.A;
                    int r = sldColorBrsh.Color.R;
                    int g = sldColorBrsh.Color.G;
                    int b = sldColorBrsh.Color.B;

                    return Get_Argb_SolidColorBrush(a, r, g, b);
                }
                else
                {
                    return "Brushes." + value;
                }
            }
            catch (Exception e)
            {
                Logging.Instance.AddMessage("Fehler im Brush Konverter mit dem Wert=" + value + ";" + e.Message);
            }

            return null;
        }


        private static string Get_ScArgb_SolidColorBrush(string a, string r, string g, string b)
        {
            return "new SolidColorBrush(Color.FromScRgb(" + a + "," + r + "," + g + "," + b + "))";
        }

        private static string Get_Argb_SolidColorBrush(int a, int r, int g, int b)
        {
            return "new SolidColorBrush(Color.FromArgb(" + a + "," + r + "," + g + "," + b + "))";
        }
    }
}
