﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.Statements;
using System.Windows.Media;

namespace Konverter.AttribValueConverters
{
    /// <summary>
    /// Konvertiert eine in Xaml definierte Color in C#-Code.
    /// </summary>
    class AttColorConverter
    {
        /// <summary>
        /// Von Color benötigte C#-Usings.
        /// </summary>
        public static List<UsingStmt> Usings
        {
            get
            {
                return new List<UsingStmt>() {
                    new UsingStmt("System.Windows.Media")
                };
            }
        }

        /// <summary>
        /// Konvertiere die in Xaml definierte Color in C#-Code.
        /// </summary>
        /// <param name="value">Xaml-Code der Color</param>
        /// <returns>C#-Code der Color</returns>
        public static string Convert(string value)
        {
            try
            {
                //RGB, ARGB -> ScRGB, ScARGB -> sonst "Red" usw.
                if (value.IndexOf("sc#") >= 0)
                {
                    BrushConverter brConv = new BrushConverter();
                    SolidColorBrush sldColorBrsh = (SolidColorBrush)brConv.ConvertFrom(value);

                    string a = sldColorBrsh.Color.ScA.ToString(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"));
                    string r = sldColorBrsh.Color.ScR.ToString(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"));
                    string g = sldColorBrsh.Color.ScG.ToString(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"));
                    string b = sldColorBrsh.Color.ScB.ToString(System.Globalization.CultureInfo.CreateSpecificCulture("en-us"));

                    return Get_ScArgb_Color(a, r, g, b);
                }
                else if (value.IndexOf("#") >= 0)
                {
                    Color color = (Color)ColorConverter.ConvertFromString(value);
                    int a = color.A;
                    int r = color.R;
                    int g = color.G;
                    int b = color.B;
                    
                    return GetColor(a, r, g, b);
                }
                else
                {
                    return "Colors." + value;
                }
            }
            catch(Exception e)
            {
                Logging.Instance.AddMessage("Fehler im Color Konverter mit dem Wert=" + value + ";" + e.Message );
            }

            return null;
        }

        private static string GetColor(int a, int r, int g, int b)
        {
            return "Color.FromArgb(" + a + "," + r + "," + g + "," + b + ")";
        }

        private static string Get_ScArgb_Color(string a, string r, string g, string b)
        {
            string c = "Color.FromScRgb(" + a + "," + r + "," + g + "," + b + "))";
            return c;
        }
    }
}
