﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.NodeGenerators;
using Konverter.Statements;

namespace Konverter.CodeTree
{
    /// <summary>
    /// Baumknoten des Typs "ElementNode" spiegeln sich im erstellten C#-Code als eigenständige Objekte wider.
    /// Eine Ausnahme davon stellen PropertyElementNode's dar, mehr dazu in der entsprechenden Klassen-Doku.
    /// </summary>
    interface ElementNode : CSTreeNode
    {
        /// <summary>
        /// Name der erstellten Variable im C#-Code.
        /// </summary>
        string VarName { get; }
        
        /// <summary>
        /// Elternknoten im CSTree.
        /// </summary>
        ElementNode Parent { get; }

        /// <summary>
        /// NodeGenerator, der für diesen ElementNode zuständig ist.
        /// </summary>
        NodeGenerator NodeGenerator { get; }

        /// <summary>
        /// Zur Erstellung des C#-Codes aus diesem Baumknoten und rekursiv aus den Kindknoten.
        /// </summary>
        /// <returns>C#-Code des Unterbaumes dieses Knotens</returns>
        CSCodeStorage CreateCode();
    }
}
