﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace Konverter
{
    /// <summary>
    /// Implementiert den Datentyp "Menge" über eine Liste.
    /// Die meisten Methoden werden dabei direkt auf List abgebildet und sind hier nicht weiter dokumentiert.
    /// </summary>
    /// <typeparam name="T">Typ der Mengen-Elemente</typeparam>
    class ListSet<T> : Set<T>
    {
        /// <summary>
        /// Liste als Speicherstruktur der Mengendaten.
        /// </summary>
        public List<T> baseList = new List<T>();

        public ListSet() { }
        public ListSet(IEnumerable<T> items)
        {
            AddRange(items);
        }

        public IEnumerator<T> GetEnumerator()
        {
            return baseList.GetEnumerator();
        }

        IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return baseList.GetEnumerator();
        }

        /// <summary>
        /// Fügt ein Element der Menge hinzu. Die Operation wird nur ausgeführt, wenn sich das Element
        /// noch nicht in der Menge befindet.
        /// </summary>
        /// <param name="item">Hinzuzufügendes Element</param>
        public void Add(T item)
        {
            if (!baseList.Contains(item))
                baseList.Add(item);
        }

        /// <summary>
        /// Fügt mehrere Elemente der Menge hinzu. Ein Element wird nur dann hinzugefügt,
        /// wenn es sich noch nicht in der Menge befindet.
        /// </summary>
        /// <param name="items"></param>
        public void AddRange(IEnumerable<T> items)
        {
            foreach (T item in items)
                Add(item);
        }

        public void Clear()
        {
            baseList.Clear();
        }

        public bool Contains(T item)
        {
            return baseList.Contains(item);
        }

        public void CopyTo(T[] array, int arrayIndex)
        {
            baseList.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get { return baseList.Count; }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public bool Remove(T item)
        {
            return baseList.Remove(item);
        }

        public int IndexOf(T item)
        {
            return baseList.IndexOf(item);
        }

        public void Insert(int index, T item)
        {
            baseList.Insert(index, item);
        }

        public void RemoveAt(int index)
        {
            baseList.RemoveAt(index);
        }

        public T this[int index]
        {
            get
            {
                return baseList[index];
            }
            set
            {
                baseList[index] = value;
            }
        }
    }
}
