﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Controls;

namespace Konverter
{
    /// <summary>
    /// Klasse für das Speichern von Logging-Nachrichten.
    /// </summary>
    class Logging
    {
        #region Event handler Definition

        public delegate void MessageAddedEventHandler(List<String> messages);
        public event MessageAddedEventHandler MessageAddedEvent;
        
        #endregion

        # region Statische Singleton Mitglieder

        private static Logging instance=null;
       
        public static Logging Instance
        { 
            get{
                if(instance==null) instance=new Logging();
                return instance;
            }
        }
        #endregion

        // Liste von Nachrichten, die im Log gespeichert werden sollen
        private List<String> messages = new List<String>();

        /// <summary>
        /// Liste der im Log gespeicherten Nachrichten.
        /// </summary>
        public List<String> Messages
        {
            get { return messages; }
        }

        /// <summary>
        /// Fügt die übergebene Nachricht zur Nachrichtenliste hinzu und wirft ein Event
        /// an alle verbundenen Abonnenten von "MessageAddedEvent".
        /// </summary>
        /// <param name="msg">Hinzuzufügende Nachricht</param>
        public void AddMessage(String msg)
        {
            //Meldungen speichern
            messages.Add(msg);

            // Event auslösen
            if (MessageAddedEvent != null)
            {
                MessageAddedEvent(messages);
            }
        }

        /// <summary>
        /// Leert das Log.
        /// </summary>
        public void ClearLog()
        {
            messages.Clear();

            // Event auslösen
            if (MessageAddedEvent != null)
            {
                MessageAddedEvent(messages);
            }
        }
    }
}
