﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.IO;

namespace Konverter
{
    /// <summary>
    /// Filter für Xaml-Code, aus dem "störende" (weil nicht übersetzbare) Codeteile entfernt werden.
    /// </summary>
    class XamlFilter
    {
        /// <summary>
        /// Optimiert das erste Kind des XmlDocuments, indem dort definierte unerwünschte Attribute
        /// entfernt werden.
        /// </summary>
        /// <param name="Xaml">Xaml-Code</param>
        /// <returns>XmlReader-Objekt des neuen Xaml-Codes</returns>
        public XmlReader OptimizeFirstChild(String Xaml)
        {
            //Xaml = Xml
            XmlDocument docXaml = new XmlDocument();
            
            StringReader stringXaml = new StringReader(Xaml);
            docXaml.Load(stringXaml);

            //NamespaceManager und Array der Namespaces erstellen
            XmlNamespaceManager nsMgr = new XmlNamespaceManager(docXaml.NameTable);
            XmlAttribute[] arrNamespace = new XmlAttribute[docXaml.FirstChild.Attributes.Count];

            int i = 0;
            foreach (XmlAttribute att in docXaml.FirstChild.Attributes)
            {
                if (att.Name.StartsWith("xmlns:"))
                {
                    string prefix = att.Name.Substring(6);
                    if (prefix != "") 
                    {
                       // prefixe[j++] = prefix;
                    }
                    nsMgr.AddNamespace(prefix, att.Value);
                    arrNamespace[i++] = att;
                }
                else if (att.Name.StartsWith("xmlns"))
                {
                    string prefix = att.Name.Substring(5);
                    nsMgr.AddNamespace(prefix, att.Value);
                    arrNamespace[i++] = att;
                }
                else if (att.Name.IndexOf(':') > 0 && att.Name.IndexOf("Class") >= -1)
                {
                    arrNamespace[i++] = att;
                }
            }

            //beliebige prefixe von ":Class" entfernen
            foreach (XmlAttribute att in arrNamespace)
            {
                if (att.Prefix != "" && att.Prefix != "xmlns")
                {
                    //XPath Knotenliste aus Namespacemanager mit "x:Class" erstellen und anschließend vorhandene Attribute löschen
                    XmlNodeList nodes = docXaml.SelectNodes("/*[@" + att.Prefix  + ":Class]", nsMgr);
                    foreach (XmlNode node in nodes)
                    {
                        node.Attributes.Remove(node.Attributes[att.Prefix + ":Class"]);
                    }
                }
            }

 

            //XMLDocument speichern als Stream, dann XmlReader erzeugen
            MemoryStream xmlStream = new MemoryStream();
            docXaml.Save(xmlStream);
            xmlStream.Position = 0;

            XmlReader xmlRead = XmlReader.Create(xmlStream);
            
            return xmlRead;
        }
    }
}
