﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using System.IO; //für StringReader
using System.Xml; //für XMLReader
using System.Windows.Markup; //für XamlReader.Load
using System.Windows.Forms; //für Datei Öffnen Dialog
using System.Windows.Threading;
using Konverter.CodeTree;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Reflection;
using System.CodeDom; //für unbehandelte Exceptions

namespace Konverter
{
    /// <summary>
    /// Interaktionslogik für Main.xaml
    /// </summary>
    public partial class Main : Window
    {
        private UIElement uie = null;
        private XmlReader xmlReader;

        // Beispielcode, der in der Textbox initial gesetzt werden soll
        private string exampleCode =
            "<Grid xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\" xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\" >\r\n"
          + "  <StackPanel Width=\"150\" Height=\"150\">\r\n"
          + "    <StackPanel.Background>\r\n"
          + "      <LinearGradientBrush>\r\n"
          + "        <LinearGradientBrush.StartPoint>0.0,0.0</LinearGradientBrush.StartPoint>\r\n"
          + "        <LinearGradientBrush.EndPoint>0.0,0.5</LinearGradientBrush.EndPoint>\r\n"
          + "        <GradientStop Offset=\"0.0\" Color=\"Blue\" />\r\n"
          + "        <GradientStop Offset=\"1.0\">\r\n"
          + "          <GradientStop.Color>Gray</GradientStop.Color>\r\n"
          + "        </GradientStop>\r\n"
          + "      </LinearGradientBrush>\r\n"
          + "    </StackPanel.Background>\r\n"
          + "    <TextBox IsReadOnly=\"TRUE\" Width=\"130\" HorizontalAlignment=\"Left\">Hallo XamlKonverter :-)</TextBox>\r\n"
          + "    <Button HorizontalAlignment=\"Right\" Name=\"Button1\" Width=\"75\" Height=\"28\" VerticalAlignment=\"Top\">\r\n"
          + "      Laden\r\n"
          + "    </Button>\r\n"
          + "    <Button HorizontalAlignment=\"Right\">\r\n"
          + "      <Button.Width>75</Button.Width>\r\n"
          + "      <Button.Height>28</Button.Height>\r\n"
          + "      <Button.Background>\r\n"
          + "        <SolidColorBrush>Red</SolidColorBrush>\r\n"
          + "      </Button.Background>\r\n"
          + "      Speichern\r\n"
          + "    </Button>\r\n"
          + "  </StackPanel>\r\n"
          + "</Grid>";


        public Main()
        {
            InitializeComponent();
            //Xaml-Code Feld initialisieren
            InitTextbox();
            
            //Handler für unbehandelte Ausnahmen initialisieren.
            Dispatcher.UnhandledException += DispatcherOnUnhandledException;

            // mit MessageAddedEvent der Logging-Klasse verbinden
            Logging.Instance.MessageAddedEvent += new Logging.MessageAddedEventHandler(MessageAdded);

            comboBoxStatus.Items.Add("Hier kommen die Meldungen rein...");
            comboBoxStatus.SelectedIndex = 0;
        }

        /// <summary>
        /// Wenn Logging ein MessageAddedEvent wirft, werden die Logging-Nachrichten in der Combobox ausgegeben.
        /// </summary>
        /// <param name="msg">Auszugebende Meldungen</param>
        void MessageAdded(List<String> messages)
        {
            comboBoxStatus.Items.Clear();
            foreach (string msg in messages)
                this.comboBoxStatus.Items.Add(msg);
            if(messages != null && messages.Count > 0)
                comboBoxStatus.SelectedItem = messages[messages.Count - 1];
        }

        /// <summary>
        /// Ereignis wird ausgelöst, wenn auf Button "Laden" geklickt wird
        /// </summary>
        private void BtnLaden_Click(object sender, RoutedEventArgs e)
        {
            // Alte Nachrichten und alten Code löschen
            Logging.Instance.ClearLog();
            TboxCSharp.Clear();

            //Existierende WPF-Elemente entfernen
            if (uie != null)
            {
                UIVorschau.Children.Remove(uie);
                uie = null;
            }
            
            if (TboxXaml.Text != "")
            {
                //XAML-Optimierungen 
                //Erst Xml, sonst exit -> dann uie, sonst -> Optimieren, dann -> uie, sonst -> Abbruch)
                try
                {
                    //Prüfen ob XML-Struktur
                    bool xmlCheck = CheckXML(TboxXaml.Text);
                    if (xmlCheck != false)
                    {
                        //UIE erstellen
                        StringReader stringReader = new StringReader(TboxXaml.Text);
                        xmlReader = XmlReader.Create(stringReader);
                        uie = GetXamlElement(xmlReader, false);

                        //Xaml optimieren, wenn UIE nicht erstellt werden konnte
                        if (uie == null)
                        {
                            //Xaml erst optimieren, dann UIE erstellen
                            XamlFilter xFilter = new XamlFilter();
                            xmlReader = xFilter.OptimizeFirstChild(TboxXaml.Text);
                            uie = GetXamlElement(xmlReader);

                            if (uie != null)
                            {
                                UIVorschau.Children.Add(uie);
                                //TODO: Das geht aber ist bescheiden, da doppelt -> xmlReader kann nur vorwärts gelesen werden
                                //xmlReader = xFilter.OptimizeFirstChild(TboxXaml.Text);
                                string UIString = XamlWriter.Save(uie);
                                stringReader = new StringReader(UIString);
                                xmlReader = XmlReader.Create(stringReader);
                                TboxCSharp.Text = GetCSharpCode(xmlReader);
                            }
                        }
                        else
                        {
                            UIVorschau.Children.Add(uie);

                            // C# Code generieren
                            stringReader = new StringReader(TboxXaml.Text);
                            xmlReader = XmlReader.Create(stringReader);
                            TboxCSharp.Text = GetCSharpCode(xmlReader);
                        }
                    }
                }
                catch (XmlException ex)
                {
                    Logging.Instance.AddMessage("Fehler beim Laden des XAML-Codes: " + ex.Message);
                    
                }
            }
            else
            {
                Logging.Instance.AddMessage("Es ist kein XAML-Code im Textfeld eingegeben worden!");
            }
        }

        /// <summary>
        /// C#-Code kompilieren und kompiliertes Fenster anzeigen.
        /// </summary>
        private void BtnShowWindow_Click(object sender, RoutedEventArgs e)
        {
            CompilerParameters compArgs = new CompilerParameters();
            compArgs.GenerateExecutable = false;

            foreach (Assembly asm in AppDomain.CurrentDomain.GetAssemblies())
                compArgs.ReferencedAssemblies.Add(asm.Location);

            CompilerResults ret = CodeDomProvider.CreateProvider("C#").CompileAssemblyFromSource(compArgs, TboxCSharp.Text);
            if (ret.Errors.Count > 0)
            {
                string errString = ret.Errors[0].ToString();
                Logging.Instance.AddMessage("Fehler beim Kompilieren: " + errString);
            }
            else
            {
                Window w = (Window)ret.CompiledAssembly.CreateInstance("GUI");
                if (null != w)
                    w.Show();
                else
                    Logging.Instance.AddMessage("Fehler: Es wurde kein Window-Objekt erzeugt.");
            }
        }

        /// <summary>
        /// UnhandledException-Handler wird für den Fall benötigt, dass das XamlObjekt eine Ausnahme auslöst
        /// </summary>
        void DispatcherOnUnhandledException(object sender, DispatcherUnhandledExceptionEventArgs args)
        {
            Logging.Instance.AddMessage("Unbehandelte Ausnahme: " + args.Exception.Message);
            args.Handled = true;
        }

        /// <summary>
        /// Prüft ob übergebener XAML-Code als XML Dokument geladen werden kann.
        /// </summary>
        /// <param name="XamlString">XAML String</param>
        /// <returns>True oder False</returns>
        private bool CheckXML(String XamlString)
        {
            //Xaml = XML; Prüfen ob XML
            try
            {
                XmlDocument docXaml = new XmlDocument();
                StringReader stringReader = new StringReader(XamlString);
                docXaml.Load(stringReader);
            }
            catch (XmlException ex)
            {
                Logging.Instance.AddMessage(ex.Message);
                return false;
            }
            finally{}

            return true;
        }
                    
        /// <summary>
        /// Liest einen XAML-Code und erzeugt daraus C#-Code
        /// </summary>
        /// <param name="xmlReader">XmlReader für den XAML Code</param>
        /// <returns>C# Code</returns>
        /// <remarks>Erstellt am 03.11.2007 von M.Jauernig</remarks>
        private string GetCSharpCode(XmlReader xmlReader)
        {
            XmlDocument xdoc = new XmlDocument();
            xdoc.Load(xmlReader);
            
            // Baum erzeugen und Code aus ihm zurückgeben
            CSTree tree = new CSTree(xdoc);
            return tree.GetCSCode();
        }

        /// <summary>
        /// Die Funktion liest einen XAML-Text ein und gibt eine Oberfläche zurück
        /// </summary>
        /// <param name="xmlReader">XmlReader für den XAML Code</param>
        /// <returns>UIElement des XAML Codes</returns>
        /// <remarks>Erstellt am 01.11.2007 von C.Kuhn</remarks>
        private UIElement GetXamlElement(XmlReader xmlReader)
        {
            return GetXamlElement(xmlReader, true);
        }
        
        private UIElement GetXamlElement(XmlReader xmlReader, bool addMessages)
        {
            try
            {
                uie = (UIElement)XamlReader.Load(xmlReader);
            }
            catch (XamlParseException e)
            {
                if(addMessages)
                    Logging.Instance.AddMessage(e.Message);
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show("Fehler beim Lesen des XAML-Codes! " + ex.Message);
            }

            return uie;
        }

        /// <summary>
        /// Initialisiert die Xaml-Code Textbox
        /// </summary>
        private void InitTextbox()
        {
            TboxXaml.Text = "";
            TboxXaml.AppendText(exampleCode);
        }

        /// <summary>
        /// Das Ereignis wird ausgelöst, wenn auf "Datei/Öffnen" geklickt worden ist
        /// </summary>
        /// <remarks>Erstellt am 01.11.2007 von C.Kuhn</remarks>
        private void MenuItemÖffnen_Click(object sender, RoutedEventArgs e)
        {
            Stream myStream;

            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.InitialDirectory = "c:\\";
            openFileDialog1.Filter = "XAML-Dateien (*.xaml)|*.xaml|XML-Dateien (*.xml)|*.xml";

            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                if ((myStream = openFileDialog1.OpenFile()) != null)
                {
                    StreamReader reader = new StreamReader(myStream);
                    TboxXaml.Clear();
                    TboxXaml.Text = reader.ReadToEnd();

                    myStream.Close();
                }
            }

        }

        /// <summary>
        /// Das Ereignis wird ausgelöst, wenn auf Datei/Beenden geklickt worden ist
        /// </summary>
        private void MenuItemBeenden_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
