﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.CodeTree;
using Konverter.ConverterBase;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "Animatable" abstammen.
    /// </summary>
    abstract class BaseGenAnimatable : NodeGenerator
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenAnimatable() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Behandlung der Attribute, die einem Animatable-Objekt eigen sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Meldungen ins Log schreiben, wenn Attribute nicht behandelt werden konnten
            foreach (ElemAttribute attrib in unhandled)
                Logging.Instance.AddMessage("Noch keine Attributbehandlung für: " + attrib.Name);

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            return ("Animatable" == tag);
        }
    }
}
