﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.CodeTree;
using Konverter.ConverterBase;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "Brush" abstammen.
    /// </summary>
    abstract class BaseGenBrush : BaseGenAnimatable
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenBrush() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Behandlung der direkt in Brush definierten Attribute
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //double
                    case "Opacity":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttDoubleConverter.Convert(attrib.Value)));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (Animatable)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("Brush" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
