﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "GradientBrush" abstammen.
    /// </summary>
    abstract class BaseGenGradientBrush : BaseGenBrush
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenGradientBrush() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();
            
            // Behandlung der Attribute eines GradientBrush
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {

                    case "ColorInterpolationMode":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttColorInterpolationModeConverter.Convert(attrib.Value), AttColorInterpolationModeConverter.Usings));
                        break;

                    case "MappingMode":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBrushMappingModeConverter.Convert(attrib.Value), AttBrushMappingModeConverter.Usings));
                        break;

                    case "SpreadMethod":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttGradientSpreadMethodConverter.Convert(attrib.Value), AttGradientSpreadMethodConverter.Usings));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (Brush)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("GradientBrush" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
