﻿using System.Collections.Generic;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;
using Konverter.Statements;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "HeaderedContentControl" abstammen.
    /// </summary>
    abstract class BaseGenHeaderedContentControl : BaseGenContentControl
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenHeaderedContentControl() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Verarbeitung der Attribute, die direkt für ButtonBase definiert sind
            foreach (ElemAttribute attrib in attribs)
            {

                switch (attrib.Name)
                {
                    //object
                    case "Header":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttObjectConverter.Convert(attrib.Value)));
                        break;

                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (ContentControl)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("HeaderedContentControl" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
