﻿using Konverter.ConverterBase;
using System.Collections.Generic;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;
using Konverter.Statements;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "ItemsControl" abstammen.
    /// </summary>
    abstract class BaseGenItemsControl : BaseGenControl, ChildElementsAddHandler
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenItemsControl() : base()
        {
            collectionAttribs.Add("Items");
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Verarbeitung der Attribute, die direkt für ButtonBase definiert sind
            foreach (ElemAttribute attrib in attribs)
            {

                switch (attrib.Name)
                {
                    
                    //bool
                    case "IsTextSearchEnabled":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;

                    //string
                    case "DisplayMemberPath":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttStringConverter.Convert(attrib.Value), AttClickModeConverter.Usings));
                        break;

                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (ContentControl)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("ItemsControl" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }

        List<Statement> ChildElementsAddHandler.GenerateAddCode(ChildElementNode parent, List<ChildElementNode> children)
        {
            List<Statement> addCode = new List<Statement>();
            foreach (ChildElementNode child in children)
            {
                addCode.Add(new MethodCall(parent.VarName + ".Items.Add", child.VarName));
            }
            return addCode;
        }
    }
}
