﻿using Konverter.ConverterBase;
using System.Collections.Generic;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "Selector" abstammen.
    /// </summary>
    abstract class BaseGenSelector : BaseGenItemsControl
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenSelector() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Verarbeitung der Attribute, die direkt für ButtonBase definiert sind
            foreach (ElemAttribute attrib in attribs)
            {

                switch (attrib.Name)
                {
                    
                    //bool
                    case "IsSynchronizedWithCurrentItem":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;

                    //int
                    case "SelectedIndex":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttIntConverter.Convert(attrib.Value)));
                        break;
                    
                    //object
                    case "SelectedItem":
                    case "SelectedValue":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttObjectConverter.Convert(attrib.Value)));
                        break;

                    //string
                    case "SelectedValuePath":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttStringConverter.Convert(attrib.Value), AttClickModeConverter.Usings));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (ContentControl)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("Selector" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
