﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.BaseGenerators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "TextBoxBase" abstammen.
    /// </summary>
    abstract class BaseGenTextBoxBase : BaseGenControl, TextAsAttribHandler
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public BaseGenTextBoxBase() : base()
        {
            
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        protected override List<PropertyValueNode> GenerateBaseNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Verarbeitung von Attributen, die direkt für jedes TextBoxBase definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //bool
                    case "AcceptsReturn":
                    case "AcceptsTab":
                    case "AutoWordSelection":
                    case "IsReadOnly":
                    case "IsUndoEnabled":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;
                    
                    //int
                    case "UndoLimit":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttIntConverter.Convert(attrib.Value)));
                        break;

                    //ScrollBarVisibility
                    case "HorizontalScrollBarVisibility":
                    case "VerticalScrollBarVisibility":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttScrollBarVisibilityConverter.Convert(attrib.Value), AttScrollBarVisibilityConverter.Usings));
                        break;

                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (Control)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if ("TextBoxBase" == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }

        public string GetTextAttribName()
        {
            return "Text";
        }

    }
}
