﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.CodeTree;
using Konverter.Statements;

namespace Konverter.NodeGenerators
{
    /// <summary>
    /// Dieses Interface wird von solchen Node-Generatoren implementiert, die in Xaml
    /// Kindelemente besitzen dürfen, denen im C#-Code also weitere Elemente hinzugefügt werden können.
    /// Dieses Interface sollte in der NodeGenerator-Hierarchie soweit oben wie möglich implementiert werden,
    /// sodass nicht verschiedene Node-Generatoren nicht denselben Code definieren müssen.
    /// Somit können Redundanzen weitestgehend vermieden werden.
    /// </summary>
    interface ChildElementsAddHandler
    {
        /// <summary>
        /// Diese Methode ist dafür zuständig, einen Vaterknoten und die Liste seiner Kind-Elementknoten
        /// entgegen zu nehmen und C#-Code in Form von Anweisungen (Statements) zurückzugeben, welche die
        /// in den Kindern definierten Variablen über eine Operation wie "Items.Add()" der Variablen
        /// im Vater hinzufügen.
        /// </summary>
        /// <param name="parent">Vaterknoten</param>
        /// <param name="children">Liste der Kind-Elementknoten des Vaters</param>
        /// <returns>Hinzuüge-Code der Kinder-Variablen zur Vater-Variable</returns>
        List<Statement> GenerateAddCode(ChildElementNode parent, List<ChildElementNode> children);
    }
}
