﻿using System.Collections.Generic;
using Konverter.NodeGenerators.BaseGenerators;
using Konverter.CodeTree;
using Konverter.ConverterBase;
using Konverter.Statements;

namespace Konverter.NodeGenerators.Generators
{
    class GenComboBoxItem : GenListBoxItem
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenComboBoxItem() : base()
        {
            tagName = "ComboBoxItem";           // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "comboBoxItem";       // Basisname für Variablen des Typs "ComboBoxItem"
            
            // benötigte Usings setzen
            usings = new List<UsingStmt>(){ new UsingStmt("System.Windows.Controls") };
        }

        /// <summary>
        /// Generiert aus den übergebenen Attributen eine Liste von Baumknoten des CSTree,
        /// welche die Attributwerte beinhalten.
        /// </summary>
        /// <param name="attribs">Zu verarbeitende Attribute</param>
        /// <returns>Generierte Baumknoten-Liste</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            // in CheckBox sind keine Properties definiert, also gleich weiterreichen
            return base.GenerateBaseNodes(attribs);
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
