﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.NodeGenerators.BaseGenerators;
using Konverter.CodeTree;
using Konverter.ConverterBase;
using Konverter.Statements;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.Generators
{
    /// <summary>
    /// Konkreter NodeGenerator zum Behandeln von Baumknoten des CSTree,
    /// die einen GradientStop repräsentieren.
    /// </summary>
    class GenGradientStop : BaseGenAnimatable
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenGradientStop() : base()
        {
            tagName = "GradientStop";            // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "gradientStop";        // Basisname für Variablen des Typs "GradientStop"

            // benötigte Usings setzen
            usings = new List<UsingStmt>() { new UsingStmt("System.Windows.Media") };
        }

        /// <summary>
        /// Generiert aus den übergebenen Attributen eine Liste von Baumknoten des CSTree,
        /// welche die Attributwerte beinhalten.
        /// </summary>
        /// <param name="attribs">Zu verarbeitende Attribute</param>
        /// <returns>Generierte Baumknoten-Liste</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Behandlung aller Attribute, welche direkt in GradientStop definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //double
                    case "Offset":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttDoubleConverter.Convert(attrib.Value)));
                        break;

                    //Color
                    case "Color":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttColorConverter.Convert(attrib.Value), AttColorConverter.Usings));
                        break;

                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Übergabe der noch unbehandelten Attribute an die Basisklasse und Verarbeitung von dieser
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
