﻿using Konverter.ConverterBase;
using System.Collections.Generic;
using Konverter.CodeTree;
using Konverter.Statements;
using Konverter.NodeGenerators.BaseGenerators;

namespace Konverter.NodeGenerators.Generators
{
    /// <summary>
    /// Konkreter NodeGenerator zum Behandeln von Baumknoten des CSTree,
    /// die eine GroupBox repräsentieren.
    /// </summary>
    class GenGroupBox : BaseGenHeaderedContentControl
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenGroupBox() : base()
        {
            tagName = "GroupBox";                  // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "groupBox";              // Basisname für Variablen des Typs "GroupBox"
            
            // benötigte Usings setzen
            usings = new List<UsingStmt>(){ new UsingStmt("System.Windows.Controls") };
        }

        /// <summary>
        /// Generiert aus den übergebenen Attributen eine Liste von Baumknoten des CSTree,
        /// welche die Attributwerte beinhalten.
        /// </summary>
        /// <param name="attribs">Zu verarbeitende Attribute</param>
        /// <returns>Generierte Baumknoten-Liste</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();
            
            // hier werden alle Attribute behandelt, die direkt in Button definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Übergabe der noch unbehandelten Attribute an die Basisklasse und Verarbeitung von dieser
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
