﻿using Konverter.ConverterBase;
using System.Collections.Generic;
using Konverter.CodeTree;
using Konverter.AttribValueConverters;
using Konverter.NodeGenerators.BaseGenerators;
using Konverter.Statements;

namespace Konverter.NodeGenerators.Generators
{
    /// <summary>
    /// Abstrakte Klasse für einen NodeGenerator, der Baumknoten für Elemente erzeugt,
    /// die vom Typ "ListBoxItem" abstammen.
    /// </summary>
    class GenListBoxItem : BaseGenContentControl
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenListBoxItem() : base()
        {
            tagName = "ListBoxItem";                  // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "listBoxItem";              // Basisname für Variablen des Typs "ListBoxItem"
            
            // benötigte Usings setzen
            usings = new List<UsingStmt>(){ new UsingStmt("System.Windows.Controls") };
        }

        /// <summary>
        /// Erzeugt Baumknoten für die übergebenen Attribute, welche hier behandelt werden sollen.
        /// </summary>
        /// <returns>Erzeugte Baumknoten</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();

            // Verarbeitung der Attribute, die direkt für ButtonBase definiert sind
            foreach (ElemAttribute attrib in attribs)
            {

                switch (attrib.Name)
                {
                    
                    //bool
                    case "IsSelected":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;

                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // rekursive Code-Erzeugung durch die Basisklasse (ContentControl)
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }

    }
}
