﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Konverter.Statements;
using Konverter.ConverterBase;
using Konverter.CodeTree;
using Konverter.NodeGenerators.BaseGenerators;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.Generators
{
    /// <summary>
    /// Konkreter NodeGenerator zum Behandeln von Baumknoten des CSTree,
    /// die ein StackPanel repräsentieren.
    /// </summary>
    class GenStackPanel : BaseGenPanel
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenStackPanel() : base()
        {
            tagName = "StackPanel";              // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "stackPanel";          // Basisname für Variablen des Typs "StackPanel"

            // benötigte Usings setzen
            usings = new List<UsingStmt>() { new UsingStmt("System.Windows.Controls") };
        }

        /// <summary>
        /// Generiert aus den übergebenen Attributen eine Liste von Baumknoten des CSTree,
        /// welche die Attributwerte beinhalten.
        /// </summary>
        /// <param name="attribs">Zu verarbeitende Attribute</param>
        /// <returns>Generierte Baumknoten-Liste</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();
            
            // Behandlung aller Attribute, welche direkt in StackPanel definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    //bool
                    case "CanHorizontallyScroll":
                    case "CanVerticallyScroll":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttBoolConverter.Convert(attrib.Value)));
                        break;

                    //Orientation
                    case "Orientation":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttOrientationConverter.Convert(attrib.Value), AttOrientationConverter.Usings));
                        break;
                    
                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Übergabe der noch unbehandelten Attribute an die Basisklasse und Verarbeitung von dieser
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
