﻿using System;
using System.Collections.Generic;
using System.Text;
using Konverter.NodeGenerators.BaseGenerators;
using Konverter.CodeTree;
using Konverter.ConverterBase;
using Konverter.Statements;
using Konverter.AttribValueConverters;

namespace Konverter.NodeGenerators.Generators
{
    class GenTabControl : BaseGenSelector
    {
        /// <summary>
        /// Konstruktor.
        /// </summary>
        public GenTabControl() : base()
        {
            tagName = "TabControl";           // Name des Tags setzen, für das der Generator verantwortlich sein soll
            varBaseName = "tabControl";       // Basisname für Variablen des Typs "TabControl"
            
            // benötigte Usings setzen
            usings = new List<UsingStmt>(){ new UsingStmt("System.Windows.Controls") };
        }

        /// <summary>
        /// Generiert aus den übergebenen Attributen eine Liste von Baumknoten des CSTree,
        /// welche die Attributwerte beinhalten.
        /// </summary>
        /// <param name="attribs">Zu verarbeitende Attribute</param>
        /// <returns>Generierte Baumknoten-Liste</returns>
        public override List<PropertyValueNode> GenerateAttribNodes(List<ElemAttribute> attribs)
        {
            List<PropertyValueNode> nodes = new List<PropertyValueNode>();
            List<ElemAttribute> unhandled = new List<ElemAttribute>();
            
            // hier werden alle Attribute behandelt, die direkt in Button definiert sind
            foreach (ElemAttribute attrib in attribs)
            {
                switch (attrib.Name)
                {
                    case "TabStripPlacement":
                        nodes.Add(new PropertyValueNode(attrib.Name, AttDockConverter.Convert(attrib.Value), AttDockConverter.Usings));
                        break;

                    default:
                        unhandled.Add(attrib);
                        break;
                }
            }

            // Übergabe der noch unbehandelten Attribute an die Basisklasse und Verarbeitung von dieser
            nodes.AddRange(base.GenerateBaseNodes(unhandled));

            return nodes;
        }

        /// <summary>
        /// Bestimmt, ob ein Tag mit dem angegebenen Namen in der Hierarchie dieses
        /// NodeGenerator's vorhanden ist.
        /// </summary>
        /// <param name="tag">Gesuchtes Tag</param>
        /// <returns>true, wenn Tag in Hierarchie vorhanden; false sonst</returns>
        public override bool HasTagInHierarchy(string tag)
        {
            if (this.tagName == tag)
                return true;
            return base.HasTagInHierarchy(tag);
        }
    }
}
