﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Konverter.NodeGenerators
{
    /// <summary>
    /// Diese Schnittstelle wird von solchen Node-Generatoren implementiert, die für Elemente zuständig sind,
    /// welche in Xaml in einem Knoten XmlText enthalten können und diesen als Attribut des umschließenden
    /// Knotens behandeln sollen. Ein Beispiel dafür wäre <Knoten>Text</Knoten>.
    /// In diesem Fall muss der Name des Xaml-Attributs definiert sein, an welches der Text übergeben werden soll.
    /// Dieses Interface sollte in der NodeGenerator-Hierarchie soweit oben wie möglich implementiert werden,
    /// um Redundanzen bei konkreten Node-Generatoren zu vermeiden.
    /// </summary>
    interface TextAsAttribHandler
    {
        /// <summary>
        /// Gibt den Namen des Xaml-Attributs zurück, an welches der XmlText weiter gereicht werden soll,
        /// das also für ihn zuständig ist.
        /// </summary>
        /// <returns>Name des Xaml-Attributs, welches durch den XmlText gesetzt wird</returns>
        string GetTextAttribName();
    }
}
