﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Konverter.NodeGenerators
{
    /// <summary>
    /// Diese Schnittstelle wird von solchen Node-Generatoren implementiert, die für Elemente zuständig sind,
    /// welche in Xaml in einem Knoten XmlText enthalten können und diesen als eigenständigen Kindknoten des 
    /// umschließenden Knotens mit genau einem Attribut behandeln sollen. Ein Beispiel dafür wäre 
    /// <Knoten>Text</Knoten>. In diesem Fall muss der Name des Xaml-Attributs definiert sein, an welches der 
    /// Text übergeben werden soll. Dieses Interface sollte in der NodeGenerator-Hierarchie soweit oben wie 
    /// möglich implementiert werden, um Redundanzen bei konkreten Node-Generatoren zu vermeiden.
    /// </summary>
    interface TextAsChildItemHandler
    {
        /// <summary>
        /// Gibt den Namen des Kindknotens zurück, welcher den XmlText als Attributwert gesetzt bekommen soll.
        /// </summary>
        /// <returns>Name des Xaml-Attributs, welches durch den XmlText gesetzt wird</returns>
        string GetItemName();

        /// <summary>
        /// Gibt den Namen des Xaml-Attributs des Kindknotens zurück, in welchem der XmlText als
        /// Wert gesetzt werden soll.
        /// </summary>
        /// <returns>Name des Xaml-Attributs, in dem der XmlText als Wert gesetzt werden soll</returns>
        string GetAttribName();
    }
}
