﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Konverter.Statements
{
    /// <summary>
    /// Anweisung, die einen Methodenaufruf darstellt.
    /// </summary>
    class MethodCall : Statement
    {
        /// <summary>
        /// Name der aufzurufenden Methode als String.
        /// </summary>
        string method;

        /// <summary>
        /// Argumente der Methode als Stringliste.
        /// </summary>
        List<string> args;

        /// <summary>
        /// Konstruktor für einen Methodenaufruf mit keinen Argumenten.
        /// </summary>
        /// <param name="method">Name der aufzurufenden Methode</param>
        public MethodCall(string method)
        {
            this.method = method;
            this.args = new List<String>();
        }

        /// <summary>
        /// Konstruktor für einen Methodenaufruf mit nur einem Argument.
        /// </summary>
        /// <param name="method">Name der aufzurufenden Methode</param>
        /// <param name="arg">An die Methode zu übergebendes Argument</param>
        public MethodCall(string method, string arg)
        {
            this.method = method;
            this.args = new List<String>() { arg };
        }

        /// <summary>
        /// Konstruktor für einen Methodenaufruf mit mehreren Argumenten.
        /// </summary>
        /// <param name="method">Name der aufzurufenden Methode</param>
        /// <param name="args">An die Methode zu übergebende Argumente</param>
        public MethodCall(string method, List<string> args)
        {
            this.method = method;
            this.args = args;
        }

        /// <summary>
        /// Gibt den Code des Methodenaufrufs zurück.
        /// </summary>
        /// <returns>Code des Methodenaufrufs</returns>
        protected override string GetCodeString()
        {
            string a = default(string);
            // erzeuge string für die zu übergebenden Argumente
            foreach (string arg in args)
            {
                // setze Komma, wenn nicht erstes Element
                if (a != default(string))
                    a += ", ";
                a += arg;
            }

            return (method + "(" + a + ")");
        }
    }
}
