﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Konverter.Statements
{
    /// <summary>
    /// Abstrakte Klasse für eine zu speichernde Anweisung.
    /// Kapselt den erzeugten Code dadurch, dass z.B. das Semikolon am Ende einer Anweisung 
    /// nicht dauernd neu geschrieben werden muss. Besser als direkt auf Strings als Code zu arbeiten.
    /// </summary>
    abstract class Statement
    {
        /// <summary>
        /// Konkrete Unterklassen haben diese Methode zu implementieren.
        /// Sie erzeugt den Code einer Anweisung als String ohne abschließendes Semikolon.
        /// </summary>
        /// <returns></returns>
        protected abstract string GetCodeString();

        /// <summary>
        /// ToString() = ToCode()
        /// </summary>
        /// <returns>Erzeugter Code der Anweisung</returns>
        public override string ToString()
        {
            return ToCode();
        }

        /// <summary>
        /// Erzeugt den Code der Anweisung. Jede Anweisung endet bei uns mit einem Semikolon.
        /// </summary>
        /// <returns>Erzeugter Code der Anweisung</returns>
        public string ToCode()
        {
            return GetCodeString() + ";";
        }
    }
}
