﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Konverter.Statements
{
    /// <summary>
    /// Anweisung, welche eine using-Direktive kapselt.
    /// </summary>
    class UsingStmt : Statement, IComparable<UsingStmt>, IEquatable<UsingStmt>
    {
        /// <summary>
        /// Gespeichertes using.
        /// </summary>
        string usg = "";

        /// <summary>
        /// Konstruktor.
        /// </summary>
        /// <param name="usg">Zu speicherndes using ohne Schlüsselwort "using" und Semikolon</param>
        public UsingStmt(string usg)
        {
            this.usg = usg;
        }

        /// <summary>
        /// Erzeugt den Code der using-Direktive.
        /// </summary>
        /// <returns>Erzeugter Code</returns>
        protected override string GetCodeString()
        {
            return "using " + usg;
        }

        /// <summary>
        /// Vergleich mit einem anderen UsingStmt. Benötigt zur Sortierung von usings.
        /// </summary>
        /// <param name="other">Zu vergleichendes using</param>
        /// <returns>Vergleichsergebnis</returns>
        public int CompareTo(UsingStmt other)
        {
            // Bilde auf gespeicherten string ab.
            return (this.usg.CompareTo(other.usg));
        }

        /// <summary>
        /// Prüfe Übereinstimmung mit einem anderen UsingStmt. Benötigt zur Sortierung von usings.
        /// </summary>
        /// <param name="other">Zu vergleichendes using</param>
        /// <returns>true, wenn die usings identisch sind (gleicher String)</returns>
        public bool Equals(UsingStmt other)
        {
            return (CompareTo(other) == 0);
        }
    }
}
